require_relative "gore"

class Enemy < ControllableObject
  attr_accessor :template, :hp

  def initialize(template)
    super(template[:material])
    @template = template
    if template[:pictures]
      @picture = template[:pictures].sample
    else
      @picture = template[:picture]
    end

    @skill = template[:skill][:class].new(self, template[:skill])
    if template[:fixed_hp]
      @hp = template[:hp]
    else
      @hp = rand(template[:hp] * 2)
    end
    @fear = rand
  end

  def chaotic
    if !@tx || rand < 0.01
      @tx = self.x + (rand - 0.5) * 500
      @ty = self.y + (rand - 0.5) * 500
    end
    self.target = vec2(@tx - x, @ty - y)
  end

  def retarget
    @targetenemy = $app.player
  end

  def hidden
    false
  end

  def update
    super
    if @template[:temporary]
      @hp -= 1.0 / 60.0
      @dead = true if @hp < 0
    end
    return if @dead
    retarget if (!@targetenemy) || @targetenemy.dead || (@targetenemy.pos - pos).length > 2000

    if (!@targetenemy) || @targetenemy.hidden || (@targetenemy.pos - pos).length > 2000
      if !@tx && @targetenemy
        @tx = @targetenemy.x
        @ty = @targetenemy.y
      end
      chaotic
      return
    end
    @tx = nil

    dx = @targetenemy.x - x
    dy = @targetenemy.y - y
    range = vec2(dx, dy).length

    mdx, mdy = dx, dy
    mdx, mdy = -mdx, mdy if range < @template[:optimal_range]

    projected = vec2(mdx, mdy) + @targetenemy.speed * MAT_ENEMY[:leash_speed] / range * @fear
    @skill.update
    @skill.apply(dx, dy, :down) if range < @skill.maxdistance * 1.5
    self.target = projected
  end

  def die
    $game.statistics[@template[:big]] += 1 unless @template[:temporary]
    $app.objects << Corpse.new(self, false) unless @template[:temporary]
    super
  end

  def side
    SIDE_ENEMY
  end

  def draw
    scale = @template[:big] ? 2 : 1
    RES::All.draw_frame(@picture, x, y, OBJSIZE / SPRITESIZE * scale, OBJSIZE / SPRITESIZE * scale)
    if @template[:boss] || @skill.class == SummonSkill
      angle = @skill.progress * 2.0 * Math::PI
      4.times do
        RES::All.draw_frame(TILE_MARK, x + Math.cos(angle) * 128, y + Math.sin(angle) * 128, OBJSIZE / SPRITESIZE, OBJSIZE / SPRITESIZE, 90 - angle * 180 / Math::PI)
        angle += Math::PI / 2
      end
    end
  end

  def take_hit(attack, fromx, fromy)
    @hp -= attack[:damage]
    if $app.player.passive.class == Empathy
      $app.player.passive.react(attack)
    end
    @dead = @hp < 0
    unless attack[:blocks]
      1.times do
        $app.objects << BloodHit.new(self, fromx, fromy)
      end
    end
  end
end
