require_relative "./lib/nonoengine"
require_relative "./resources"
require_relative "./gameobject"
require_relative "./physics_wrap"
require_relative "./physics"
require_relative "./player"
require_relative "./map"
require_relative "./mapgen/mapgen"
require_relative "./bullet"
require_relative "./enemy"
require_relative "./story"

class GameLevel < GameApp
  attr_accessor :physics
  attr_accessor :player
  attr_accessor :map
  attr_accessor :objects
  attr_accessor :broken_doors
  attr_accessor :explosions
  attr_accessor :slow_motion
  attr_accessor :boss

  def spawn_enemy(template)
    enemy = Enemy.new(template)
    pt = @map.random_point(LogicalTile::Passage)
    pt = @map.portal if template == ENEMY_SUPERBOSS
    enemy.warp(pt[0] * TILESIZE, pt[1] * TILESIZE)
    @objects << enemy
    enemy
  end

  def new_player(klass)
    #@boss.hp = [@boss.template[:hp], @boss.hp + 20].min if @boss
    if @player
      @player.dead = true
      @objects.delete player
      @player.die
    end
    @gamma_timer = 0
    @player = Player.new(klass)
    pt = @map.entry
    @player.warp(pt[0] * TILESIZE, pt[1] * TILESIZE)
    @objects << @player
    @slow_motion = false
    @slow_tick = 0
  end

  def existing_player(guy)
    #@boss.hp = [@boss.template[:hp], @boss.hp + 20].min if @boss
    if @player
      @player.dead = true
      @objects.delete player
      @player.die
    end
    @player = guy.clone_for_level
    @gamma_timer = 0
    pt = @map.entry
    @player.warp(pt[0] * TILESIZE, pt[1] * TILESIZE)
    @objects << @player
    @slow_motion = false
    @slow_tick = 0
    @objects.select { |v| v.class == Bullet }.each { |v| v.dead = true }
  end

  def activate
    $app = self
  end

  def initialize(gen)
    super()
    activate

    @physics = PhysicEngine.new
    @physics.register Player
    @physics.register Walls
    @physics.register Door
    @physics.register Stair
    @physics.register Bullet
    @physics.register Enemy
    @physics.register Golem
    @physics.register Corpse
    @physics.end_registration

    @map = Map.new(gen)
    @objects << @map

    if gen.class == TownGenerator
      massacre((gen.palace_door[0] + 0.5) * TILESIZE, (gen.palace_door[1] + 5) * TILESIZE)
    end

    N_MONSTERS[gen.class].times do
      spawn_enemy(gen_monster(USE_MONSTERS[gen.class]))
    end
    @boss = spawn_enemy(ENEMY_SUPERBOSS) if gen.class == DungeonGenerator

    @broken_doors = []
    @explosions = []

    @font = RES::Vera
    @font.config(color: Colors::YELLOW)
  end

  DIR_KEYS = {
    KEY_W => [0, -1],
    KEY_S => [0, 1],
    KEY_A => [-1, 0],
    KEY_D => [1, 0],
    KEY_UP => [0, -1],
    KEY_DOWN => [0, 1],
    KEY_LEFT => [-1, 0],
    KEY_RIGHT => [1, 0],
  }

  def update
    v = 19 * 60 - $game.time.to_i
    $game.story.game_over if v < 0
    dx = Mouse[Mouse::X] - SCREENX / 2
    dy = Mouse[Mouse::Y] - SCREENY / 2
    #player.angle = Math.atan2(dy, dx)

    step = [0, 0]
    DIR_KEYS.each do |k, v|
      if Engine::Key[k] != :up
        step[0] += v[0]
        step[1] += v[1]
      end
    end
    player.target = vec2(100 * step[0], 100 * step[1])

    @broken_doors.uniq.each(&:open)
    @broken_doors = []

    @explosions.uniq.each(&:explode)
    @explosions = []

    spawn_enemy(ENEMY_CIVILIAN) if $game.story.storypoint == :at_portal && rand < 1.0 / 60 / 10

    @slow_motion = false if @player.hp > @player.pain
    if @slow_motion && @slow_tick < SLOWMO_SCALE
      @player.update
      @slow_tick += 1
    else
      if @player.hp < @player.pain
        @gamma_timer += 1
        $game.story.player_died if @gamma_timer > DEATH_TIME
      else
        @gamma_timer = 0
      end

      @slow_tick = 0
      @physics.update
      super
    end
  end

  def draw
    # RES::Space.background
    if DEBUG_MAP
      camera(0, 0)
    else
      camera(@player.x - SCREENX / 2, @player.y - SCREENY / 2)
    end
    super
    Engine.layer = 100
    player.show_hud

    if @gamma_timer > 0
      Engine.rect2(0, 0, SCREENX, SCREENY, true, Engine::Colors.new(55, 0, 0, 100 + 155 * (@gamma_timer / DEATH_TIME)))
    else
      relative_pain = (PAIN_THRESHOLD - (@player.hp - @player.pain) / PLAYER_HP) / PAIN_THRESHOLD
      if relative_pain > 0 && @passive.class != NoPain && @player.pain > 0.1
        Engine.rect2(0, 0, SCREENX, SCREENY, true, Engine::Colors.new(55, 0, 0, relative_pain * 100))
      end
    end

    v = 19 * 60 - $game.time.to_i
    min = (v / 60).to_s.rjust(2, "0")
    sec = (v % 60).to_s.rjust(2, "0")
    @font.draw_text("Осталось времени: #{min}:#{sec}", 10, 10)
    @font.draw_text("Убито демонов: #{$game.statistics[true]}, прислужников #{$game.statistics[nil]}", 10, 30)
  end
end
