
class DungeonGenerator < MapGenerator
  attr_accessor :entry
  attr_accessor :portal

  def self.get_size
    100
  end

  def gen_entry
    (0...@size).each do |ny|
      (0...@size / 2).each do |dx|
        2.times do |sx|
          y = @size - 1 - ny
          x = (sx == 0 ? -1 : 1) * dx + @size / 2
          if @tile_data[ny * @size + x] == Bearlib::TILE_BIGROOM
            return [x, ny]
          end
        end
      end
    end
  end

  def gen_portal
    bestd = 0
    best = @entry
    200.times do
      nx = 0
      ny = 0
      loop do
        nx = rand(@size)
        ny = rand(@size)
        break if @tile_data[ny * @size + nx] == Bearlib::TILE_BIGROOM
      end
      d = (nx - @entry[0]).abs + (ny - @entry[1]).abs
      if d > bestd
        best = [nx, ny]
        bestd = d
      end
    end
    best
  end

  def generate(data)
    bearmap = Bearlib::Map.new(@size, @size, {cells: :integer})
    layer = bearmap.layers[:cells]
    @tile_data = Array.new(@size * @size)
    @wave_data = Array.new(@size * @size)
    loop do
      layer.generate(Bearlib::G_NICE_DUNGEON, SEED)
      (@size * @size).times do |i|
        x = i % @size
        y = i / @size
        @tile_data[y * @size + x] = layer[x, y]
      end
      next unless check_space
      next unless check_connected
      break
    end
    fix_walls
    double_size if @double
    @entry = gen_entry
    @portal = gen_portal

    make_from_tile_data data

    data[(@portal[1]) * @size + @portal[0]] = {
      pass: false,
      picture: TILE_PORTAL1,
      logic: LogicalTile::Stair,
    }

    (-1..1).each do |dx|
      (-1..1).each do |dy|
        next if dx == 0 && dy == 0
        data[(@portal[1] + dy) * @size + @portal[0] + dx] = {
          pass: true,
          picture: TILE_PORTAL2,
          logic: LogicalTile::Passage,
        }
      end
    end
  end
end
