class TownGenerator < MapGenerator
  attr_reader :palace_door
  # def initialize
  #   @double = false
  #   @tile_data = []
  #   @size = @double ? self.class.get_size / 2 : self.class.get_size
  # end

  def self.get_size
    200
  end

  def shift_terrain(data)
    data.map! do |v|
      {
        pass: v[:pass],
        picture: v[:picture] - 24,
        logic: v[:logic],
      }
    end
  end

  def add_house(x, y, w, h)
    (x - 1..x + w).each do |ax|
      (y - 1..y + h).each do |ay|
        t = tile(ax, ay)
        return if [Bearlib::TILE_CAVE_WALL, Bearlib::TILE_HOUSE_FLOOR].include? t
        #return if t == Bearlib::TILE_ROOMWALL
      end
    end

    @houses << [x, y, w, h]
    (x...x + w).each do |ax|
      (y...y + h).each do |ay|
        @tile_data[ay * @size + ax] = Bearlib::TILE_ROOMWALL
      end
    end
  end

  def add_palace(x, y, w, h)
    (x...x + w).each do |ax|
      (y...y + h).each do |ay|
        @tile_data[ay * @size + ax] = Bearlib::TILE_CAVE_WALL
      end
    end
    (x...x + w).each do |ax|
      (y + h...y + h + 5).each do |ay|
        @tile_data[ay * @size + ax] = Bearlib::TILE_HOUSE_FLOOR
      end
    end
  end

  def entry
    if $game.story.storypoint == :at_castle
      x = @palace_door.clone
      x[0] += rand(5) - 2
      x[1] += 5 + rand(4) - 2
      return x
    end
    door = @doors.pop
    $game.story.storypoint == :at_castle if @doors.size == 0
    door
  end

  def generate(data)
    loop do
      @tile_data = Array.new(@size * @size, Bearlib::TILE_SMALLROOM)
      @size.times do |i|
        @tile_data[0 * @size + i] = Bearlib::TILE_ROOMWALL
        @tile_data[(@size - 1) * @size + i] = Bearlib::TILE_ROOMWALL
        @tile_data[i * @size + 0] = Bearlib::TILE_ROOMWALL
        @tile_data[i * @size + (@size - 1)] = Bearlib::TILE_ROOMWALL
      end

      @wave_data = Array.new(@size * @size)

      @houses = []
      @doors = []
      #place a palace
      w = rand(5) + 8
      h = 8
      add_palace @size / 2 - w / 2, @size / 2 - h / 2, w, h
      @palace_door = [@size, @size + h]
      #place some houses
      (0.012 * 4 * @size * @size).to_i.times do
        w = rand(5) + 2
        h = rand(3) + 2
        x = rand(@size - w)
        y = rand(@size - h)
        add_house x, y, w, h
      end

      check_connected([@size / 2, @size / 2 + h / 2 + 3])

      #now place doors for connected houses
      @houses.each do |x, y, w, h|
        next if w < 3
        dx = x + 1 + rand(w - 2)
        next if tile(dx, y + h) != Bearlib::TILE_SMALLROOM
        next if tile(dx - 1, y + h) != Bearlib::TILE_SMALLROOM
        next if tile(dx + 1, y + h) != Bearlib::TILE_SMALLROOM
        next if @wave_data[(y + h) * @size + dx] == 0
        #@tile_data[(y + h - 1) * @size + dx] = Bearlib::TILE_DOOR
        @tile_data[(y + h) * @size + dx] = Bearlib::TILE_BIGROOM
        @tile_data[(y + h) * @size + dx - 1] = Bearlib::TILE_BIGROOM
        @tile_data[(y + h) * @size + dx + 1] = Bearlib::TILE_BIGROOM
        @doors << [dx * 2, (y + h) * 2]
      end
      break if @doors.size > 0
    end

    #fix_walls
    double_size if @double
    make_from_tile_data data
    shift_terrain data

    @doors.each do |x, y|
      data[(y - 0) * @size + x] = {
        pass: true,
        picture: TILE_BIGDOOR1,
        logic: LogicalTile::Passage,
      }
      data[(y - 1) * @size + x] = {
        pass: false,
        picture: TILE_BIGDOOR2,
        logic: LogicalTile::Wall,
      }
    end

    @doors.sort_by! { |x, y| (x - @palace_door[0]).abs + (y - @palace_door[1]).abs }

    (0..1).each do |dx|
      data[(@palace_door[1] - 2) * @size + @palace_door[0] + dx] = {
        pass: false,
        picture: TILE_GATE1,
        logic: LogicalTile::Wall,
      }
      data[(@palace_door[1] - 1) * @size + @palace_door[0] + dx] = {
        pass: false,
        picture: TILE_GATE2,
        logic: LogicalTile::Stair,
      }
      data[(@palace_door[1]) * @size + @palace_door[0] + dx] = {
        pass: true,
        picture: TILE_GATE3,
        logic: LogicalTile::Passage,
      }
    end
  end
end
