require "./skills/skills"
require_relative "./lib/nonoengine"

# Our Player
class Player < ControllableObject
  attr_accessor :hp, :pain, :left_skill, :right_skill, :passive, :dancing, :hidden, :last_touch, :golem, :picture, :klass, :traps, :blood_level

  def initialize(klass)
    super(klass[:material])
    @klass = klass
    @picture = klass[:picture]
    @picture += 1 * 8 unless klass[:single_sex] || rand < 0.5
    @at_door = nil
    @font = RES::Vera
    @font.config(color: Colors::WHITE)
    @left_skill = klass[:left_skill][:class].new(self, klass[:left_skill])
    @right_skill = klass[:right_skill][:class].new(self, klass[:right_skill])
    @passive = klass[:passive][:class].new(self, klass[:passive]) if klass[:passive]
    @hp = PLAYER_HP
    @pain = 0.0
    @blood_level = 0.0
    @traps = []
  end

  def clone_for_level
    x = Player.new(@klass)
    x.picture = @picture
    x.left_skill = @left_skill
    x.left_skill.owner = x
    x.right_skill = @right_skill
    x.right_skill.owner = x
    x.hp = @hp
    x.pain = @pain
    x.passive = @passive
    x.passive.owner = x if x.passive
    x.blood_level = @blood_level
    x
  end

  def update
    super
    @passive.update if @passive
    @pain -= PAIN_DECREMENT if @pain > 0
    @at_door = nil if @at_door && @at_door.broken
    @at_door = nil if @at_door && (mapx - @at_door.x).abs + (mapy - @at_door.y).abs > 1

    dx = Mouse[Mouse::X] - SCREENX / 2
    dy = Mouse[Mouse::Y] - SCREENY / 2
    @left_skill.update
    @right_skill.update
    if Mouse[Mouse::LEFT_BUTTON] != :up
      @left_skill.apply(dx, dy, Mouse[Mouse::LEFT_BUTTON])
    end
    if Mouse[Mouse::RIGHT_BUTTON] != :up
      @right_skill.apply(dx, dy, Mouse[Mouse::RIGHT_BUTTON])
    end

    if @at_door && Engine::Key[KEY_SPACE] != :up
      @at_door.open
      @at_door = nil
    end

    RES::Skills::Melee3.play if @dancing && !RES::Skills::Melee3.playing?
  end

  def side
    SIDE_ALLY
  end

  def draw
    if @hidden
      RES::All.draw_frame(TILE_HUNTER_HIDDEN, x, y, OBJSIZE / SPRITESIZE, OBJSIZE / SPRITESIZE)
    else
      scale = @klass[:big] ? 2 : 1
      RES::All.draw_frame(@picture, x, y, OBJSIZE / SPRITESIZE * scale, OBJSIZE / SPRITESIZE * scale)
      if @passive.class == AutoHide
        show_revertprogress(x, y, OBJSIZE / 2, @passive.progress)
      end
    end
    #RES::Slash.draw_frame(1, @control_body.p.x, @control_body.p.y, OBJSIZE / SPRITESIZE, OBJSIZE / SPRITESIZE)
  end

  def take_hit(attack, fromx, fromy)
    if @passive.class == OnlyPain
      @pain += attack[:damage] * @passive.stats[:power]
    else
      @hp -= attack[:damage] * DAM_WOUNDS
      @pain += attack[:damage] * DAM_PAIN
    end

    (attack[:damage] * BLOOD_LEVEL).to_i.times do
      $app.objects << BloodHit.new(self, fromx, fromy)
    end
  end

  def show_hud
    @font.draw_text(@at_door.message, SCREENX / 2 - 100, SCREENY / 2 - 100) if @at_door

    @left_skill.draw_hud(true)
    @right_skill.draw_hud(false)

    #@font.draw_text("left #{@left_skill.delay} right #{@right_skill.delay}", SCREENX / 2 - 100, SCREENY / 2 + 100)
    unless @passive.class == NoPain
      w = 450
      damaged = w * (PLAYER_HP - @hp) / PLAYER_HP
      pains = w * @pain / PLAYER_HP
      basex = SCREENX / 2 - w / 2
      Engine.rect2(basex, SCREENY - 50, w, 25, true, Engine::Colors.new(0, 50, 0, 50))
      Engine.rect2(basex, SCREENY - 50, damaged, 25, true, Engine::Colors.new(0, 0, 0))
      Engine.rect2(basex + damaged, SCREENY - 50, pains, 25, true, Engine::Colors.new(0xff, 0, 0))
    end

    if @golem && !@golem.dead
      w = 250
      damaged = w * (@golem.template[:hp] - @golem.hp) / @golem.template[:hp]
      basex = SCREENX / 2 - w / 2
      Engine.rect2(basex, SCREENY - 100, w, 25, true, Engine::Colors.new(0, 50, 0, 50))
      Engine.rect2(basex, SCREENY - 100, damaged, 25, true, Engine::Colors.new(0, 0, 0))
    end

    if $app.boss
      w = SCREENX - 200
      damaged = w * ($app.boss.template[:hp] - $app.boss.hp) / $app.boss.template[:hp]
      basex = SCREENX / 2 - w / 2
      Engine.rect2(basex, 10, w - damaged, 25, true, Engine::Colors.new(200, 0, 0, 150))
      #Engine.rect2(basex, 10, damaged, 25, true, Engine::Colors.new(0, 0, 0))
    end
  end

  def collide_door(door)
    @at_door = door
    true
  end

  def collide_stair(stair)
    @at_door = stair
    true
  end

  def collide_enemy(enemy)
    if @dancing && @dancing[:ignore] != enemy
      enemy.take_hit(@dancing, x, y)
    end
    @last_touch = enemy
    @hidden = false if @hidden
    true
  end
end
