class BloodShootSkill < ShootSkill
  def draw_hud(is_left)
    x = is_left ? 100 : SCREENX - 100
    y = SCREENY - 100
    r = [@owner.blood_level * 10, 100].min
    Engine.circle(x, y, r, true, 0, Engine::Colors::RED) if r > 1
    super
  end

  def apply(dx, dy, btn_state)
    return if @delay > 0
    super
    @owner.blood_level = @owner.blood_level * (1.0 - @stats[:blood_use])
  end
end

BLOODMAGE_SLASH = {
  desc: "Удар кинжалом",
  class: BloodShootSkill,
  picture: PROJ_RAY,
  maxrange: 4,
  speed: 1000,
  spread: Math::PI / 2,
  count: 1,
  delay: 15,
  #offset: Math::PI * 0.3,
  damage: 2,
  force: 5.0,
  pierce: 1,
  blood_use: 0.05,
  sound: RES::Skills::Sword,
}

class FateShootSkill < ShootSkill
  def initialize(*args)
    super
    @delay = @stats[:delay]
  end

  def update
    #don't recharge
  end

  def apply(dx, dy, btn_state)
    return if @delay > 0
    super
    @owner.hp = PLAYER_HP / 2 if @owner.hp < PLAYER_HP / 2
  end
end

BLOODMAGE_FATE = {
  desc: "Изничтожение",
  class: FateShootSkill,
  picture: 12 * 8 + 4,
  maxrange: 50,
  speed: 500,
  spread: 2 * Math::PI,
  count: 32,
  delay: 200,
  damage: 10,
  sound: RES::Skills::Lightning,
  explode: {
    picture: PROJ_FIRE,
    maxrange: 50,
    speed: 500,
    spread: 2 * Math::PI,
    count: 16,
    delay: 10,
    damage: 2,
    pierce: 3,
    sound: RES::Skills::Shot_fire,
  },

}

class BloodMagic < Skill
  @@base_dam = nil

  def initialize(*args)
    super
    @@base_dam = BLOODMAGE_SLASH[:damage] unless @@base_dam
  end

  def get_blood
    @owner.blood_level += @stats[:power]
    @owner.hp += @stats[:power] / 4 if @owner.hp < PLAYER_HP * 0.25
  end

  def update
    @owner.left_skill.stats[:damage] = @@base_dam * [1 + @owner.blood_level / 2, 1 + 10 / 2].min

    @owner.right_skill.delay -= @owner.pain / PLAYER_HP if @owner.pain > 0.02
    #@owner.pain -= PAIN_DECREMENT if @owner.pain > 0

    cur_dam = @owner.left_skill.stats[:damage]
  end
end

BLOODMAGE_DESC = <<me
кормит кинжал кровью
  использует свою боль для могущественного Изничтожения
me

BLOODMAGE =
  {
    material: MAT_PLAYER,
    picture: 13 * 8 + 4,
    name: "Кровавый маг",
    desc: BLOODMAGE_DESC,
    single_sex: true,

    left_skill: BLOODMAGE_SLASH,
    right_skill: BLOODMAGE_FATE,
    passive: {class: BloodMagic, power: 0.5},
  }
