
class SlingerShootSkill < ShootSkill
  def initialize(*args)
    super
    @ammo = @stats[:ammo]
    @reloading = false
    @last_shoot = 0
  end

  def progress
    if @reloading
      1.0 - 1.0 * @delay / @stats[:reload]
    else
      1.0 * @ammo / @stats[:ammo]
    end
  end

  def update
    super
    if @delay <= 0 && @reloading
      @reloading = false
      @ammo = @stats[:ammo]
    end
    $app.slow_motion = @owner.pain > @owner.hp
    @last_shoot += 1 unless @reloading
    if !@reloading && @last_shoot > 60 && @ammo < @stats[:ammo]
      @reloading = true
      @delay = @stats[:reload]
    end
  end

  def apply(dx, dy, btn_state)
    return unless btn_state == :clicked
    return if @ammo <= 0
    if @reloading
      @reloading = false
      @delay = 0
    end
    return if @delay > 0
    @last_shoot = 0
    x = @stats[:spread]
    @stats[:spread] = x * (1 + 40 * @owner.pain / PLAYER_HP)
    super
    @stats[:spread] = x
    if @delay > 0
      @ammo -= 1
      if @ammo <= 0
        @delay = @stats[:reload]
        @reloading = true
      end
    end
  end

  def draw_hud(is_left)
    x = is_left ? 100 : SCREENX - 100
    y = SCREENY - 100
    RES::Ammo.draw_frame(@ammo, x, y, 1, 1)
    show_progress(x, y, 50, progress.clamp(0, 1)) if @reloading
  end
end

SLINGER_SHOT = {
  desc: "Выстрел",
  class: SlingerShootSkill,
  picture: PROJ_RAY,
  maxrange: 200,
  speed: 2000,
  spread: 0.05,
  count: 1,
  delay: 10,
  damage: 7,
  force: 15.0,
  ammo: 6,
  reload: 3 * 60,
  sound: RES::Skills::Pistol,
}

SLINGER =
  {
    name: "Стрелок",
    desc: "боль снижает точность",
    material: MAT_PLAYER,
    picture: 14 * 8 + 4,
    single_sex: true,

    left_skill: SLINGER_SHOT,
    right_skill: SLINGER_SHOT,
  }
