
HUNTER_SHOT = {
  desc: "Выстрел",
  class: ShootSkill,
  picture: PROJ_RAY,
  maxrange: 60,
  speed: 1200,
  spread: 0.05,
  count: 1,
  delay: 25,
  damage: 3,
  force: 5.0,
  sound: RES::Skills::Crossbow,
}

class TrapSkill < ShootSkill
  def cant_use
    !@owner.hidden || @owner.traps.any? { |tr| (tr.pos - @owner.pos).length < OBJSIZE }
  end

  def update
    @delay = 0.0 if cant_use || (@capture && (@capture - @owner.pos).length > 0.0001)
  end

  def apply(dx, dy, btn_state)
    RES::Skills::Trapset.play if progress > 0 && progress < 1 && !RES::Skills::Trapset.playing?
    @delay += 1.0
    @capture = vec2(@owner.x, @owner.y)
    if @delay >= @stats[:delay]
      @delay = 0
      super
      @owner.hidden = true
      @delay = 0
    end
  end

  def progress
    cant_use ? 0 : super
  end

  def draw_hud(is_left)
    x = is_left ? 100 : SCREENX - 100
    y = SCREENY - 100
    pic = @stats[:hud] || (@owner.klass[:picture] + (is_left ? 5 : 6) * 8)
    RES::All.draw_frame(pic, x, y, 100 / SPRITESIZE, 100 / SPRITESIZE)
    show_revertprogress(x, y, 50, 1 - progress.clamp(0, 1))
  end
end

HUNTER_TRAP = {
  desc: "Установить ловушку",
  class: TrapSkill,
  picture: TILE_TRAP,
  maxrange: 30 * 60,
  speed: 0.0,
  spread: 0.05,
  count: 1,
  delay: 50,
  damage: 12,
  trap: 5,
  explode: {
    picture: PROJ_POISON,
    maxrange: 25,
    speed: 500,
    spread: 2 * Math::PI,
    count: 16,
    delay: 10,
    damage: 3,
    pierce: 3,
    sound: RES::Skills::Traphit,
  },
}

class AutoHide < Skill
  @@base_dam = nil

  def initialize(*args)
    super
    @delay = 0
    @capture = vec2(0, 0)
    @@base_dam = HUNTER_SHOT[:damage] unless @@base_dam
    @owner.hidden = true
  end

  def update
    dam = @@base_dam
    dam *= @stats[:hidden_scale] if @owner.hidden
    dam *= (1.0 + @stats[:pain_scale] * @owner.pain / PLAYER_HP) if @owner.pain > 0.01

    cur_dam = @owner.left_skill.stats[:damage]
    @owner.left_skill.stats[:damage] = cur_dam + (dam - cur_dam) * 0.03

    if @owner.hidden
      @delay = 0
      return
    end
    if (@capture - @owner.pos).length > 0.0001
      @delay = 0.0
      @capture = vec2(@owner.x, @owner.y)
    else
      @delay += 1.0
      if @delay >= @stats[:delay]
        @owner.hidden = true
      end
    end
  end

  def progress
    @delay / @stats[:delay]
  end
end

HUNTER_DESC = <<me
Скрывается если неподвижен
  Первые выстрелы из скрытности наносят больше урона
  Боль заставляет бить сильнее
me

HUNTER =
  {
    name: "Охотник",
    desc: HUNTER_DESC,
    material: MAT_PLAYER,
    picture: 13 * 8 + 3,

    left_skill: HUNTER_SHOT,
    right_skill: HUNTER_TRAP,
    passive: {
      class: AutoHide,
      delay: 150,
      pain_scale: 2.0,
      hidden_scale: 10.0,
    },
  }
