require_relative "../enemy"

class PainfulShootSkill < ShootSkill
  def apply(dx, dy, btn_state)
    return if @delay > 0
    super
    if @delay > 0
      @owner.pain += @stats[:pain]
    end
  end
end

WARLOCK_FIREBALL = {
  desc: "Огненный шар",
  class: PainfulShootSkill,
  pain: 1.0,
  picture: PROJ_FIRE,
  maxrange: 40,
  speed: 1000,
  spread: 0.1,
  count: 1,
  delay: 25,
  damage: 5,
  sound: RES::Skills::Shot_fire,
  explode: {
    picture: PROJ_FIRE,
    maxrange: 5,
    speed: 500,
    spread: 2 * Math::PI,
    count: 12,
    delay: 10,
    damage: 1,
    pierce: 2,
    sound: RES::Skills::Explode,
  },
}

GOLEM = {
  picture: 7 * 8 + 4,
  hp: 20,
  optimal_range: 0.0,
  fixed_hp: true,

  skill: {
    class: ShootSkill,
    picture: PROJ_SLASH,
    maxrange: 10,
    speed: 200,
    spread: 0.6,
    count: 3,
    delay: 60,
    damage: 4,
    force: 200.0,
    sound: RES::Skills::Amelee,
  },
  material: {
    mass: 4.0,
    inertia: 4.0,
    elast: 0.1,
    friction: 0.1,
    leash_speed: 400.0,
    leash_force: 4000.0,
  },
}

class Golem < Enemy
  def initialize(*args)
    super
    @fear = 0
    @owner = $app.player
  end

  def retarget
    @targetenemy = $app.objects.select { |u| u.class == Enemy }.min_by { |u| (u.pos - @owner.pos).length }
  end

  def update
    super
    retarget if rand < 1.0 / 60
  end

  def side
    SIDE_ALLY
  end
end

class SummonGolemSkill < Skill
  def initialize(*args)
    super
    @delay = 0
  end

  def apply(dx, dy, btn_state)
    return if @delay > 0
    @owner.golem.dead = true if @owner.golem
    @delay = stats[:delay]
    x = owner.x + rand - 0.5
    y = owner.y + rand - 0.5
    golem = Golem.new(GOLEM)
    golem.warp(x, y)
    $app.objects << golem
    @owner.golem = golem
    @owner.pain = @owner.hp if @owner.pain < @owner.hp
    @stats[:sound].play if @stats[:sound]
  end

  def progress
    1.0 - 1.0 * @delay / @stats[:delay]
  end

  def update
    @delay -= 1 if @delay > 0
  end
end

WARLOCK =
  {
    name: "Чернокнижник",
    desc: "заклинания вызывают боль",
    material: MAT_PLAYER,
    picture: 13 * 8 + 0,

    left_skill: WARLOCK_FIREBALL,
    right_skill: {class: SummonGolemSkill, delay: 10 * 60, desc: "Вызов голема", sound: RES::Skills::Curse},
  }
